"""Commons."""


extensions = dict(JPEG="jpg", PNG="png", GTiff="tif", WEBP="webp")

drivers = dict(jpg="JPEG", png="PNG", tif="GTiff", webp="WEBP")

mimetype = dict(
    png="image/png",
    jpg="image/jpg",
    webp="image/webp",
    pbf="application/x-protobuf",
    mvt="application/x-protobuf",
)
